#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2010, Sylvain Raybaud <sylvain.raybaud@loria.fr>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@loria.fr> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

from optparse import OptionParser
from mytools import open_plain_or_gz,warn,error,verbose_msg,openfiles,closefiles,getline
import sys
from time import time
#from string import join
from re import sub
from PyQt4 import QtGui, QtCore
from threading import Timer
from os import getenv
from random import randrange
from levenshtein import levenshtein

ICONPATH = "postedit-icons/"
start_message = """The quick way:
 - click 'start training' in the menu or toolbar to start a predefined training session (if available)
 - Use the slider to set the requested translation quality. You can change that anytime you want.
 - than click 'start xp1' and 'start xp2' to start predefined experiments (if available).
 - You have to click 'reset' (nuke) between each session
 - You can do the training session as many times you want, but do the experiments only once. Any subsequent run is worthless.
 - If you want to take a break, click 'stop' and possibly save your session ot restore it later. Don't just leave the program running, it would make time statistics meaningless.

The detailed way:
 - Use the menu and toolbar to open your own:
   * file containing source sentences
   * file containing translation hypothesis
   * file containing confidence scores (optional)
   * file for saving reordered source sentences (optional, but useful if input is randomized)
   * file for saving edited hypothesis
   * file for saving time statistics (optional)
 - Again, use the slider to set the requested translation quality
 - Above recommendations about reseting and taking breaks apply."""

#current_dir = getenv('HOME')
current_dir = getenv('PWD')

class CurrentHyp:
    def __init__(self,src,hyp,index):
        self.src = src
        self.hyp = hyp
        self.index = index
        self.typing = 0.0
        self.start_time = time()
        self.end_time = -9999.0
        self.timer = None
        self.typing_started = 9999.0
        self.sliding_started = 9999.0
        self.sliding_time = 0
        self.n_wrong = 0
        self.src_len = len(self.src.split())
        self.hyp_len = len(self.hyp.split())

class Data:
    def __init__(self):
        self.srcfilename = None
        self.srcoutfilename = None
        self.hypfilename = None
        self.hyppefilename = None
        self.cmfilename = None
        self.outfilename = None
        self.src = []
        self.scores = []
        self.hyp = []
        self.sentences_ids = []
        self.min = 1
        self.max = 0

    def norm(self,x):
        return (x-self.min)/(self.max-self.min)

    def load(self):
        srcfile = open_plain_or_gz(self.srcfilename,"r")
        self.src = srcfile.readlines()
        #for s in srcfile.readlines():
            ##self.src.append(unicode(s,"utf8"))
            #self.src.append(s.decode("utf8"))
        srcfile.close()
        hypfile = open_plain_or_gz(self.hypfilename,"r")
        self.hyp = hypfile.readlines()
        #for h in hypfile.readlines():
            ##self.hyp.append(unicode(h,"utf8"))
            #self.hyp.append(h.decode("utf8"))
        hypfile.close()
        if self.cmfilename:
            cmfile = open_plain_or_gz(self.cmfilename)
            self.scores = []
            for h in self.hyp:
                h_scores = []
                for w in h.split():
                    score = float(getline(cmfile).split(',')[0])
                    if score<self.min:
                        self.min = score
                    if score>self.max:
                        self.max = score
                    h_scores.append(score)
                self.scores.append(h_scores)
            cmfile.close()
        self.sentences_ids = list(range(0,len(self.src)))

    def write_out_header(self):
        if self.outfilename:
            out = open_plain_or_gz(self.outfilename,'w')
            out.write("#src=%s\n" % self.srcfilename)
            out.write("#hyp=%s\n" % self.hypfilename)
            out.write("#cm=%s\n" % self.cmfilename)
            out.write("#pair_id,total_time,thinking_time,typing_time,interface_time,src_length,hyp_length,edited_hyp_length,cm_init_wrong_words,edit_distance\n")
            out.close()
        if self.hyppefilename:
            out = open_plain_or_gz(self.hyppefilename,'w')
            out.write("#src=%s\n" % self.srcfilename)
            out.write("#hyp=%s\n" % self.hypfilename)
            out.write("#cm=%s\n" % self.cmfilename)
            out.close()
        if self.srcoutfilename:
            out = open_plain_or_gz(self.srcoutfilename,'w')
            out.write("#src=%s\n" % self.srcfilename)
            out.write("#hyp=%s\n" % self.hypfilename)
            out.write("#cm=%s\n" % self.cmfilename)
            out.close()

class MyQTextEdit(QtGui.QTextEdit):
    def event(self,event):
        if event.type()==QtCore.QEvent.KeyPress:
            self.parent().keypressed()
        return QtGui.QTextEdit.event(self,event)

class MyQSlider(QtGui.QSlider):
    def mousePressEvent(self,event):
        self.parent().mousePressed()
        return QtGui.QSlider.mousePressEvent(self,event)
    def mouseReleaseEvent(self,event):
        self.parent().mouseReleased()
        return QtGui.QSlider.mouseReleaseEvent(self,event)

class MainWindow(QtGui.QMainWindow):

    def __init__(self, data, opts, win_parent = None):

        self.data = data
        self.status = "init"
        self.do_random = True

        #Init the base class
        QtGui.QMainWindow.__init__(self, win_parent)
        self.setWindowTitle("Post Editor")
        self.create_widgets()
        #self.set_layout()
        self.resize(800, 400)
        self.statusBar()

    #def set_layout(self):

    def save(self):
        if self.status=="running":
            self.statusBar().showMessage("Finish current sentence and click 'stop' first.")
            return
        fname = QtGui.QFileDialog.getSaveFileName(self, 'Choose file to save current state',current_dir)
        if fname=='':
            return
        f = open_plain_or_gz(fname,'w')
        f.write("src=%s\n" % self.data.srcfilename)
        f.write("hyp=%s\n" % self.data.hypfilename)
        f.write("cm=%s\n" % self.data.cmfilename)
        f.write("pe=%s\n" % self.data.hyppefilename)
        f.write("srcout=%s\n" % self.data.srcoutfilename)
        f.write("stats=%s\n" % self.data.outfilename)
        f.write("do_random=%i\n" % int(self.do_random))
        f.write("todo=%s\n" % str(self.data.sentences_ids))
        f.close()
        return

    def load(self):
        global current_dir
        if self.status!="init":
            self.statusBar().showMessage("To late to restore previous session!")
            error_popup("To late to restore previous session! You must finish first and then click 'reset'.")
            return
        fname = QtGui.QFileDialog.getOpenFileName(self, 'Restore session from file...',current_dir)
        if fname=='':
            return
        current_dir = sub("/[^/]*$","",str(fname))
        self.load_from_file(fname)

    def start_training(self):
        if self.status!="init":
            self.statusBar().showMessage("To late to start training!")
            error_popup("To late to start training session! You must finish first and then click 'reset'.")
            return
        try:
            self.load_from_file(current_dir+"/train.pe")
        except:
            self.statusBar().showMessage("'train.pe' not found!")
            error_popup("Can't open file 'train.pe' in directory '%s'. Check existence and permissions or open it using the 'restore session' dialog." % current_dir)
            #error_window = QtGui.QWidget()
            #QtGui.QMessageBox.critical(error_window, "Error!",message, QtGui.QMessageBox.Ok)
            return
        self.statusBar().showMessage('Ready to start training session.')
        info_popup("Training session loaded. Click 'Start'. When you are done, click 'Reset' and load another session.")
        self.data.write_out_header()
        return

    def start_xp1(self):
        if self.status!="init":
            self.statusBar().showMessage("To late to start xp1!")
            error_popup("To late to start experiment! You must finish first and then click 'reset'.")
            return
        try:
            self.load_from_file(current_dir+"/xp1.pe")
        except:
            self.statusBar().showMessage("'xp1.pe' not found!")
            error_popup("Can't open file 'xp1.pe' in directory '%s'. Check existence and permissions or open it using the 'restore session' dialog." % current_dir)
            #error_window = QtGui.QWidget()
            #QtGui.QMessageBox.critical(error_window, "Error!",message, QtGui.QMessageBox.Ok)
            return
        self.statusBar().showMessage('Ready to start XP #1.')
        info_popup("First experiment loaded. Click 'Start'. When you are done, click 'Reset' and load another session.")
        self.data.write_out_header()
        return

    def start_xp2(self):
        if self.status!="init":
            self.statusBar().showMessage("To late to start xp2!")
            error_popup("To late to start experiment! You must finish first and then click 'reset'.")
            return
        try:
            self.load_from_file(current_dir+"/xp2.pe")
        except:
            self.statusBar().showMessage("'xp2.pe' not found!")
            error_popup("Can't open file 'xp2.pe' in directory '%s'. Check existence and permissions or open it using the 'restore session' dialog." % current_dir)
            #error_window = QtGui.QWidget()
            #QtGui.QMessageBox.critical(error_window, "Error!",message, QtGui.QMessageBox.Ok)
            return
        self.statusBar().showMessage('Ready to start XP #2.')
        info_popup("Second experiment loaded. Click 'Start'. When you are done, click 'Reset' and load another session.")
        self.data.write_out_header()
        return

    def reset(self):
        self.data = Data()
        self.main_widget.data = self.data
        self.disconnect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.start)
        self.disconnect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.restart)
        self.disconnect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.next)
        self.connect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.start)
        self.main_widget.next_button.setText("&Start")
        self.main_widget.next_button.setIcon(QtGui.QIcon(ICONPATH+"start.png"))
        self.main_widget.next_button.setDisabled(False)
        self.main_widget.stop_button.setDisabled(True)
        self.main_widget.src_content.setText('')
        self.main_widget.hyp_content.setText('')
        self.main_widget.hyp_content.setDisabled(True)
        self.main_widget.pair_id.display(0)
        self.status = "init"
        return

    def load_from_file(self,fname):
        f = open_plain_or_gz(fname,'r')
        lines = f.readlines()
        f.close()
        todo = None
        for l in lines:
            if l!='' and l[0]!='#' and l[0]!='\n':
                lsplit = l.strip().split("=")
                var = lsplit[0]
                val = lsplit[1]
                if var=="src" and val!="None":
                    self.data.srcfilename = val
                elif var=="hyp" and val!="None":
                    self.data.hypfilename = val
                elif var=="cm" and val!="None":
                    self.data.cmfilename = val
                elif var=="pe" and val!="None":
                    self.data.hyppefilename = val
                elif var=="srcout" and val!="None":
                    self.data.srcoutfilename = val
                elif var=="stats" and val!="None":
                    self.data.outfilename = val
                elif var=="do_random":
                    self.do_random = bool(int(val))
                    self.main_widget.random_check.setCheckState(self.do_random)
                elif var=="todo":
                    todo = []
                    val = val.strip("[]").split(',')
                    for v in val:
                        todo.append(int(v))
        self.data.load()
        if todo!=None:
            self.data.sentences_ids = todo
        self.disconnect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.start)
        self.disconnect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.restart)
        self.disconnect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.next)
        self.connect(self.main_widget.next_button,QtCore.SIGNAL('clicked()'),self.main_widget.restart)
        self.main_widget.next_button.setText("&Start")
        self.main_widget.next_button.setIcon(QtGui.QIcon(ICONPATH+"start.png"))
        self.main_widget.next_button.setDisabled(False)
        self.main_widget.stop_button.setDisabled(True)
        self.status = "stopped"
        if self.data.cmfilename:
            self.main_widget.threshold_slider.setDisabled(False)
            self.main_widget.quality_label.setDisabled(False)
            self.main_widget.quality_label.setText("Quality")
        else:
            self.main_widget.threshold_slider.setDisabled(True)
            self.main_widget.quality_label.setDisabled(True)
            self.main_widget.quality_label.setText("(no CM)")
        return

    def open_src_file(self):
        if self.status=="init":
            global current_dir
            self.data.srcfilename = QtGui.QFileDialog.getOpenFileName(self, 'Choose source sentences file',current_dir)
            verbose_msg(opts,"source file is '%s'" % self.data.srcfilename)
            current_dir = sub("/[^/]*$","",str(self.data.srcfilename))
        else:
            self.statusBar().showMessage('You can\'t change source sentences file while running!')

    def open_srcout_file(self):
        if self.status=="init":
            global current_dir
            self.data.srcoutfilename = QtGui.QFileDialog.getSaveFileName(self, 'Choose output source sentences file',current_dir)
            verbose_msg(opts,"source out file is '%s'" % self.data.srcoutfilename)
            current_dir = sub("/[^/]*$","",str(self.data.srcoutfilename))
        else:
            self.statusBar().showMessage('You can\'t change output source sentences file while running!')

    def open_hyp_file(self):
        if self.status=="init":
            global current_dir
            self.data.hypfilename = QtGui.QFileDialog.getOpenFileName(self, 'Choose hypothesis file',current_dir)
            verbose_msg(opts,"hyp file is '%s'" % self.data.hypfilename)
            current_dir = sub("/[^/]*$","",str(self.data.hypfilename))
        else:
            self.statusBar().showMessage('You can\'t change hypothesis file while running!')

    def open_hyppe_file(self):
        if self.status=="init":
            global current_dir
            self.data.hyppefilename = QtGui.QFileDialog.getSaveFileName(self, 'Choose post-edited hypothesis file',current_dir)
            verbose_msg(opts,"hyppe file is '%s'" % self.data.hyppefilename)
            current_dir = sub("/[^/]*$","",str(self.data.hyppefilename))
        else:
            self.statusBar().showMessage('You can\'t change post-edited hypothesis file while running!')

    def open_cm_file(self):
        if self.status=="init":
            global current_dir
            self.data.cmfilename = QtGui.QFileDialog.getOpenFileName(self, 'Choose confidence file',current_dir)
            verbose_msg(opts,"cm file is '%s'" % self.data.cmfilename)
            current_dir = sub("/[^/]*$","",str(self.data.cmfilename))
        else:
            self.statusBar().showMessage('You can\'t change confidence file while running!')

    def open_out_file(self):
        if self.status=="init":
            global current_dir
            self.data.outfilename = QtGui.QFileDialog.getSaveFileName(self, 'Choose output stat file',current_dir)
            verbose_msg(opts,"out file is '%s'" % self.data.outfilename)
            current_dir = sub("/[^/]*$","",str(self.data.outfilename))
        else:
            self.statusBar().showMessage('You can\'t change output file while running!')

    def create_widgets(self):
        #Widgets
        self.main_widget = MainWidget(self.data,opts,parent=self)
        self.setCentralWidget(self.main_widget)

        # Actions
        exit = QtGui.QAction(QtGui.QIcon(ICONPATH+"quit-48.png"), 'Exit', self)
        exit.setShortcut('Ctrl+Q')
        exit.setStatusTip('Exit application')
        self.connect(exit, QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()'))
        open_src = QtGui.QAction(QtGui.QIcon(ICONPATH+"open-src-48.png"), 'Open source sentences file', self)
        #open_src.setShortcut('Ctrl+Q')
        open_src.setStatusTip('Open source sentences file')
        self.connect(open_src, QtCore.SIGNAL('triggered()'), self.open_src_file)
        open_srcout = QtGui.QAction(QtGui.QIcon(ICONPATH+"open-srcout-48.png"), 'Open output source sentences file', self)
        #open_src.setShortcut('Ctrl+Q')
        open_srcout.setStatusTip('Open output source sentences file')
        self.connect(open_srcout, QtCore.SIGNAL('triggered()'), self.open_srcout_file)
        open_hyp = QtGui.QAction(QtGui.QIcon(ICONPATH+"open-hyp-48.png"), 'Open hypothesis', self)
        #open_hyp.setShortcut('Ctrl+Q')
        open_hyp.setStatusTip('Open hypothesis')
        self.connect(open_hyp, QtCore.SIGNAL('triggered()'), self.open_hyp_file)
        open_hyppe = QtGui.QAction(QtGui.QIcon(ICONPATH+"open-hyppe-48.png"), 'Open post-edited hypothesis', self)
        #open_hyppe.setShortcut('Ctrl+Q')
        open_hyppe.setStatusTip('Open post-edited hypothesis')
        self.connect(open_hyppe, QtCore.SIGNAL('triggered()'), self.open_hyppe_file)
        open_cm = QtGui.QAction(QtGui.QIcon(ICONPATH+"open-cm-48.png"), 'Open CM', self)
        #open_cm.setShortcut('Ctrl+Q')
        open_cm.setStatusTip('Open CM')
        self.connect(open_cm, QtCore.SIGNAL('triggered()'), self.open_cm_file)
        open_out = QtGui.QAction(QtGui.QIcon(ICONPATH+"open-data-48.png"), 'Open output stat file', self)
        #open_out.setShortcut('Ctrl+Q')
        open_out.setStatusTip('Choose output file')
        self.connect(open_out, QtCore.SIGNAL('triggered()'), self.open_out_file)
        save_session = QtGui.QAction(QtGui.QIcon(ICONPATH+"save-128.png"), 'Save session', self)
        save_session.setShortcut('Ctrl+S')
        save_session.setStatusTip('Save session')
        self.connect(save_session, QtCore.SIGNAL('triggered()'), self.save)
        load_session = QtGui.QAction(QtGui.QIcon(ICONPATH+"load-48.png"), 'Restore session', self)
        load_session.setShortcut('Ctrl+O')
        load_session.setStatusTip('Load session')
        self.connect(load_session, QtCore.SIGNAL('triggered()'), self.load)
        start_training = QtGui.QAction(QtGui.QIcon(ICONPATH+"train.png"), 'Start training', self)
        start_training.setShortcut('Ctrl+T')
        start_training.setStatusTip('Start training')
        self.connect(start_training, QtCore.SIGNAL('triggered()'), self.start_training)
        start_xp1 = QtGui.QAction(QtGui.QIcon(ICONPATH+"xp1.png"), 'Start first experiment', self)
        start_xp1.setShortcut('Ctrl+1')
        start_xp1.setStatusTip('Start first experiment')
        self.connect(start_xp1, QtCore.SIGNAL('triggered()'), self.start_xp1)
        start_xp2 = QtGui.QAction(QtGui.QIcon(ICONPATH+"xp2.png"), 'Start second experiment', self)
        start_xp2.setShortcut('Ctrl+2')
        start_xp2.setStatusTip('Start second experiment')
        self.connect(start_xp2, QtCore.SIGNAL('triggered()'), self.start_xp2)
        reset = QtGui.QAction(QtGui.QIcon(ICONPATH+"reset.png"), '*CAUTION* reset program (not files)', self)
        reset.setStatusTip('*CAUTION* reset program (not files)')
        self.connect(reset, QtCore.SIGNAL('triggered()'), self.reset)

        self.menubar = self.menuBar()
        self.file_menu = self.menubar.addMenu('&File')
        self.file_menu.addAction(open_src)
        self.file_menu.addAction(open_hyp)
        self.file_menu.addAction(open_cm)
        self.file_menu.addAction(open_srcout)
        self.file_menu.addAction(open_hyppe)
        self.file_menu.addAction(open_out)
        self.file_menu.addAction(save_session)
        self.file_menu.addAction(load_session)
        self.file_menu.addAction(start_training)
        self.file_menu.addAction(start_xp1)
        self.file_menu.addAction(start_xp2)
        self.file_menu.addAction(exit)
        self.file_menu.addAction(reset)
        #self.options_menu = self.menubar.addMenu('&Options')

        self.toolbar = self.addToolBar('Actions')
        self.toolbar.addAction(open_src)
        self.toolbar.addAction(open_hyp)
        self.toolbar.addAction(open_cm)
        self.toolbar.addSeparator()
        self.toolbar.addAction(open_srcout)
        self.toolbar.addAction(open_hyppe)
        self.toolbar.addAction(open_out)
        self.toolbar.addSeparator()
        self.toolbar.addAction(save_session)
        self.toolbar.addAction(load_session)
        self.toolbar.addSeparator()
        self.toolbar.addAction(start_training)
        self.toolbar.addAction(start_xp1)
        self.toolbar.addAction(start_xp2)
        self.toolbar.addSeparator()
        self.toolbar.addAction(exit)
        self.toolbar.addSeparator()
        self.toolbar.addAction(reset)


class MainWidget(QtGui.QWidget):
    def __init__(self, data, opts, parent=None):
        self.data = data
        self.opts = opts
        self.current_hyp = None
        self.cm_threshold = 0.5
        QtGui.QWidget.__init__(self, parent)
        self.create_widgets()
        self.set_layout()

    def getStatus(self):
        return self.parent().status
    def setStatus(self,status):
        self.parent().status = status
    def getRandom(self):
        return self.parent().do_random
    def setRandom(self,rand):
        self.parent().do_random = rand

    def stop(self):
        verbose_msg(opts,"'stop' clicked")
        self.data.sentences_ids.insert(0,self.current_hyp.index)
        self.setStatus("stopped")
        self.src_content.setText("")
        self.hyp_content.setText("")
        self.hyp_content.setDisabled(True)
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.start)
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.restart)
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.next)
        self.connect(self.next_button,QtCore.SIGNAL('clicked()'),self.restart)
        self.next_button.setText("&Restart")
        self.next_button.setIcon(QtGui.QIcon(ICONPATH+"start.png"))
        self.stop_button.setDisabled(True)
        self.pair_id.display(0)
        if self.current_hyp.timer:
            self.current_hyp.timer.cancel()

    def restart(self):
        verbose_msg(opts,"'restart' clicked")
        self.setStatus("running")
        if not self.load_new_hyp():
            return
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.start)
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.restart)
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.next)
        self.connect(self.next_button,QtCore.SIGNAL('clicked()'),self.next)
        self.next_button.setText("&Next")
        self.next_button.setIcon(QtGui.QIcon(ICONPATH+"next.png"))
        self.stop_button.setDisabled(False)
        self.pair_id.setDisabled(False)
        if self.data.cmfilename:
            self.threshold_slider.setDisabled(False)
            self.quality_label.setDisabled(False)
            self.quality_label.setText("Quality")
        else:
            self.threshold_slider.setDisabled(True)
            self.quality_label.setDisabled(True)
            self.quality_label.setText("(no CM)")

    def start(self):
        verbose_msg(opts,"'start' clicked")
        if self.data.srcfilename==None or self.data.hypfilename==None or self.data.hyppefilename==None:
            message = "You must specify at least a source file, an hypothesis file and a post-edited hypothesis file!"
            error_window = QtGui.QWidget()
            QtGui.QMessageBox.critical(error_window, "Error!",message, QtGui.QMessageBox.Ok)
            return
        self.data.load()
        self.data.write_out_header()
        self.stop_button.setDisabled(False)
        if not self.load_new_hyp():
            return
        self.disconnect(self.next_button,QtCore.SIGNAL('clicked()'),self.start)
        self.connect(self.next_button,QtCore.SIGNAL('clicked()'),self.next)
        self.next_button.setText("&Next")
        self.next_button.setIcon(QtGui.QIcon(ICONPATH+"next.png"))
        self.pair_id.setDisabled(False)
        if self.data.cmfilename:
            self.threshold_slider.setDisabled(False)
            self.quality_label.setDisabled(False)
            self.quality_label.setText("Quality")
        else:
            self.threshold_slider.setDisabled(True)
            self.quality_label.setDisabled(True)
            self.quality_label.setText("(no CM)")

    def next(self):
        verbose_msg(opts,"'next' clicked")
        self.current_hyp.end_time = time()
        if self.current_hyp.timer:
            self.current_hyp.timer.cancel()
            self.current_hyp.typing += self.current_hyp.end_time-self.current_hyp.typing_started
        save(self.data,self.current_hyp,self.hyp_content.toPlainText())
        self.setStatus("running")
        self.load_new_hyp()

    def load_new_hyp(self):
        if len(self.data.sentences_ids)==0:
            self.setStatus("finished")
            self.finished()
            return False
        hid = self.data.sentences_ids.pop(randrange(0,len(self.data.sentences_ids))) if self.getRandom() else self.data.sentences_ids.pop(0)
        src = self.data.src[hid]
        hyp = self.data.hyp[hid]
        self.current_hyp = CurrentHyp(src,hyp,hid)
        self.src_content.setText(src_font(src))
        self.hyp_content.setText(hyp_font(self.decorate_hyp(hyp)))
        self.hyp_content.setDisabled(False)
        self.pair_id.display(hid+1)
        return True

    def keypressed(self):
        if self.current_hyp.timer==None:
            self.current_hyp.timer = Timer(1,self.timer_ended)
            self.current_hyp.timer.start()
            self.current_hyp.typing_started = time()
        else:
            self.current_hyp.timer.cancel()
            self.current_hyp.timer = Timer(1,self.timer_ended)
            self.current_hyp.timer.start()

    def mousePressed(self):
        if self.current_hyp:
            self.current_hyp.sliding_started = time()

    def mouseReleased(self):
        if self.current_hyp:
            self.current_hyp.sliding_time += time()-self.current_hyp.sliding_started

    def timer_ended(self):
        self.current_hyp.timer = None
        self.current_hyp.typing += time()-self.current_hyp.typing_started-0.5


    def set_layout(self):

        main_layout = QtGui.QGridLayout()
        main_layout.setSpacing(10)

        main_layout.addWidget(self.random_check,0,0,1,1)
        main_layout.addWidget(self.threshold_slider,3,0,4,1)
        main_layout.addWidget(self.quality_label,7,0,1,1)
        main_layout.addWidget(self.src_title,0,3,1,1)
        main_layout.addWidget(self.hyp_title,3,3,1,1)
        main_layout.addWidget(self.src_content,0,4,3,10)
        main_layout.addWidget(self.hyp_content,3,4,4,10)
        #main_layout.addWidget(self.stop_button,7,7,1,1)
        #main_layout.addWidget(self.next_button,7,10,1,1)
        #main_layout.addWidget(self.decision_title,3,15,1,1)
        main_layout.addWidget(self.pair_id_lbl,0,15,1,1)
        main_layout.addWidget(self.pair_id,1,15,1,1)
        main_layout.addWidget(self.next_button,3,15,1,1)
        main_layout.addWidget(self.stop_button,5,15,1,1)

        #self.src_title.adjustSize()
        #self.hyp_title.adjustSize()
        #self.stop_button.adjustSize()
        #self.next_button.adjustSize()
        ##self.decision_title.adjustSize()
        #self.select_for_pe.adjustSize()
        #self.select_as_correct.adjustSize()
        #self.random_check.adjustSize()
        #self.threshold_slider.adjustSize()
        #self.quality_label.adjustSize()

        self.setLayout(main_layout)

    def create_widgets(self):
        #Widgets
        self.src_title = QtGui.QLabel("source:",self)
        self.src_content = QtGui.QTextEdit("",self)
        self.src_content.setDisabled(True)
        self.hyp_title = QtGui.QLabel("hypothesis:",self)
        self.hyp_content = MyQTextEdit("",self)
        self.hyp_content.setDisabled(True)

        self.next_button = QtGui.QPushButton(QtGui.QIcon(ICONPATH+"start.png"),"&Start")
        self.stop_button = QtGui.QPushButton(QtGui.QIcon(ICONPATH+"pause.png"),"&Pause")

        self.random_check = QtGui.QCheckBox("Randomize input",self)
        self.random_check.setCheckState(self.getRandom())
        #self.threshold_slider = QtGui.QSlider(QtCore.Qt.Vertical, self)
        self.threshold_slider = MyQSlider(QtCore.Qt.Vertical, self)
        self.threshold_slider.setMinimum(0)
        self.threshold_slider.setMaximum(1000)
        self.threshold_slider.setValue(self.cm_threshold*1000)
        self.quality_label = QtGui.QLabel("Quality",self)
        self.pair_id_lbl = QtGui.QLabel("Sentence id:",self)
        self.pair_id = QtGui.QLCDNumber(self)

        self.pair_id.display(0)
        self.stop_button.setDisabled(True)

        self.connect(self.random_check, QtCore.SIGNAL('stateChanged(int)'), self.toggle_random)
        self.connect(self.threshold_slider, QtCore.SIGNAL('valueChanged(int)'), self.change_threshold)
        self.connect(self.next_button,QtCore.SIGNAL('clicked()'),self.start)
        self.connect(self.stop_button,QtCore.SIGNAL('clicked()'),self.stop)

    def toggle_random(self):
        self.setRandom(self.random_check.isChecked())
        verbose_msg(self.opts,"'do_random' is now %s" % str(self.random_check.isChecked()))
        return

    def change_threshold(self):
        self.cm_threshold = float(self.threshold_slider.value())/1000
        verbose_msg(self.opts,"threshold is now %f" % self.cm_threshold)
        if self.data.cmfilename and self.getStatus()=="running":
            self.hyp_content.setText(hyp_font(self.decorate_hyp(undecorate(self.hyp_content.toPlainText()))))
        return

    def finished(self):
        self.parent().statusBar().showMessage("Nothing left to do.")
        self.src_content.setText("")
        self.hyp_content.setText("")
        self.hyp_content.setDisabled(True)
        self.next_button.setDisabled(True)
        self.stop_button.setDisabled(True)

    def decorate_hyp(self,hyp):
        self.current_hyp.n_wrong = 0
        if not self.data.cmfilename:
            return hyp
        hsplit = hyp.split()
        hdeco = []
        for i,w in enumerate(hsplit):
            if self.data.norm(self.data.scores[self.current_hyp.index][i])<self.cm_threshold:
                hdeco.append("<font color=\"red\">%s</font>" % w)
                self.current_hyp.n_wrong += 1
            else:
                hdeco.append(w)
        return " ".join(hdeco)


def src_font(s):
    return '<font size="05"><i>%s</i></font>' % s

def hyp_font(h):
    return '<font size="05">%s</font>' % h

def undecorate(hyppe):
    return sub("<[^>]*>","",hyppe)

def save(data,current_hyp,hyppe):
    if data.hyppefilename and hyppe:
        #hyppeout = codecs.open(data.hyppefilename,mode='a',encoding='utf8')
        hyppeout = open_plain_or_gz(data.hyppefilename,'a')
        #hyppeout.write((undecorate(str(hyppe)).strip()+u"\n").encode("utf8"))
        hyppeout.write(undecorate(str(hyppe)).strip()+"\n")
        hyppeout.close()
    if data.srcoutfilename:
        #srcout = codecs.open(data.srcoutfilename,mode='a',encoding='utf8')
        srcout = open_plain_or_gz(data.srcoutfilename,'a')
        #srcout.write((str(src).strip()+u"\n").encode("utf8"))
        srcout.write(current_hyp.src.strip()+"\n")
        srcout.close()
    if data.outfilename:
        total_time = current_hyp.end_time-current_hyp.start_time
        thinking_time = total_time-current_hyp.typing
        datout = open_plain_or_gz(data.outfilename,'a')
        edits,_,_ = levenshtein(current_hyp.hyp.split(),str(hyppe).split())
        datout.write("%i,%f,%f,%f,%f,%i,%i,%i,%i,%i\n" % (current_hyp.index,\
                                                        total_time,\
                                                        thinking_time,\
                                                        current_hyp.typing,\
                                                        current_hyp.sliding_time,\
                                                        current_hyp.src_len,\
                                                        current_hyp.hyp_len,\
                                                        len(str(hyppe).split()),\
                                                        current_hyp.n_wrong,\
                                                        edits))
        datout.close()

def error_popup(msg):
    error_window = QtGui.QWidget()
    QtGui.QMessageBox.critical(error_window, "Error!",msg, QtGui.QMessageBox.Ok)
    return

def info_popup(msg):
    information_window = QtGui.QWidget()
    QtGui.QMessageBox.information(information_window, "Information",msg, QtGui.QMessageBox.Ok)
    return

if __name__=='__main__':
    parser = OptionParser(usage="Usage: %prog [options] POSITIONAL ARGUMENTS")
    parser.add_option('-v','--verbose',default=False,action="store_true",dest="verbose",help="verbosity [%default]")
    opts,args = parser.parse_args()
    if len(args)!=0:
        parser.print_help()
        error("bad command line.",1)

    data = Data()

    main_app = QtGui.QApplication(sys.argv)
    main_window = MainWindow(data,opts)
    main_window.show()

    info_popup(start_message)

    code = main_app.exec_()
    verbose_msg(opts,"Bye bye. (application returned code %i)" % code)
    sys.exit(code)
